@@ -27,7 +27,6 @@ import scala.collection.JavaConverters._
 import org.apache.kyuubi.{Logging, Utils}
 import org.apache.kyuubi.engine.{EngineType, ShareLevel}
 import org.apache.kyuubi.service.authentication.{AuthTypes, SaslQOP}
-import org.apache.kyuubi.util.NettyUtils.MAX_NETTY_THREADS
 
 case class KyuubiConf(loadSysDefault: Boolean = true) extends Logging {
   import KyuubiConf._
@@ -232,6 +231,14 @@ object KyuubiConf {
       .checkValue(t => t > 0, "must be positive integer")
       .createWithDefault(Duration.ofMinutes(1).toMillis)
 
+  val CREDENTIALS_UPDATE_WAIT_TIMEOUT: ConfigEntry[Long] =
+    buildConf("credentials.update.wait.timeout")
+      .doc("How long to wait until credentials are ready.")
+      .version("1.5.0")
+      .timeConf
+      .checkValue(t => t > 0, "must be positive integer")
+      .createWithDefault(Duration.ofMinutes(1).toMillis)
+
   val CREDENTIALS_HADOOP_FS_ENABLED: ConfigEntry[Boolean] =
     buildConf("credentials.hadoopfs.enabled")
       .doc("Whether to renew Hadoop filesystem delegation tokens")
@@ -516,6 +523,16 @@ object KyuubiConf {
     .checkValue(p => p == 0 || (p > 1024 && p < 65535), "Invalid Port number")
     .createWithDefault(3309)
 
+  /**
+   * Specifies an upper bound on the number of Netty threads that Kyuubi requires by default.
+   * In practice, only 2-4 cores should be required to transfer roughly 10 Gb/s, and each core
+   * that we use will have an initial overhead of roughly 32 MB of off-heap memory, which comes
+   * at a premium.
+   *
+   * Thus, this value should still retain maximum throughput and reduce wasted off-heap memory
+   * allocation.
+   */
+  val MAX_NETTY_THREADS: Int = 8
   val FRONTEND_MYSQL_NETTY_WORKER_THREADS: OptionalConfigEntry[Int] =
     buildConf("frontend.mysql.netty.worker.threads")
       .doc("Number of thread in the netty worker event loop of MySQL frontend service. " +
@@ -577,6 +594,45 @@ object KyuubiConf {
       .stringConf
       .createOptional
 
+  val ENGINE_FLINK_MAIN_RESOURCE: OptionalConfigEntry[String] =
+    buildConf("session.engine.flink.main.resource")
+      .doc("The package used to create Flink SQL engine remote job. If it is undefined," +
+        " Kyuubi will use the default")
+      .version("1.4.0")
+      .stringConf
+      .createOptional
+
+  val ENGINE_FLINK_MAX_ROWS: ConfigEntry[Int] =
+    buildConf("session.engine.flink.max.rows")
+      .doc("Max rows of Flink query results. For batch queries, rows that exceeds the limit " +
+        "would be ignored. For streaming queries, the query would be canceled if the limit " +
+        "is reached.")
+      .version("1.5.0")
+      .intConf
+      .createWithDefault(1000000)
+
+  val ENGINE_TRINO_MAIN_RESOURCE: OptionalConfigEntry[String] =
+    buildConf("session.engine.trino.main.resource")
+      .doc("The package used to create Trino engine remote job. If it is undefined," +
+        " Kyuubi will use the default")
+      .version("1.5.0")
+      .stringConf
+      .createOptional
+
+  val ENGINE_TRINO_CONNECTION_URL: OptionalConfigEntry[String] =
+    buildConf("session.engine.trino.connection.url")
+      .doc("The server url that trino engine will connect to")
+      .version("1.5.0")
+      .stringConf
+      .createOptional
+
+  val ENGINE_TRINO_CONNECTION_CATALOG: OptionalConfigEntry[String] =
+    buildConf("session.engine.trino.connection.catalog")
+      .doc("The default catalog that trino engine will connect to")
+      .version("1.5.0")
+      .stringConf
+      .createOptional
+
   val ENGINE_LOGIN_TIMEOUT: ConfigEntry[Long] = buildConf("session.engine.login.timeout")
     .doc("The timeout of creating the connection to remote sql query engine")
     .version("1.0.0")
@@ -619,11 +675,12 @@ object KyuubiConf {
     .doc("The check interval for engine timeout")
     .version("1.0.0")
     .timeConf
-    .checkValue(_ >= Duration.ofSeconds(3).toMillis, "Minimum 3 seconds")
-    .createWithDefault(Duration.ofMinutes(5).toMillis)
+    .checkValue(_ >= Duration.ofSeconds(1).toMillis, "Minimum 1 seconds")
+    .createWithDefault(Duration.ofMinutes(1).toMillis)
 
   val ENGINE_IDLE_TIMEOUT: ConfigEntry[Long] = buildConf("session.engine.idle.timeout")
-    .doc("engine timeout, the engine will self-terminate when it's not accessed for this duration")
+    .doc("engine timeout, the engine will self-terminate when it's not accessed for this " +
+      "duration. 0 or negative means not to self-terminate.")
     .version("1.0.0")
     .timeConf
     .createWithDefault(Duration.ofMinutes(30L).toMillis)
@@ -663,7 +720,16 @@ object KyuubiConf {
       .checkValue(_ > 0, "the maximum must be positive integer.")
       .createWithDefault(10)
 
-  // TODO: make it true by default
+  val SESSION_ENGINE_STARTUP_WAIT_COMPLETION: ConfigEntry[Boolean] =
+    buildConf("session.engine.startup.waitCompletion")
+      .doc("Whether to wait for completion after engine starts." +
+        " If false, the startup process will be destroyed after the engine is started." +
+        " Note that only use it when the driver is not running locally," +
+        " such as yarn-cluster mode; Otherwise, the engine will be killed.")
+      .version("1.5.0")
+      .booleanConf
+      .createWithDefault(true)
+
   val SESSION_ENGINE_LAUNCH_ASYNC: ConfigEntry[Boolean] =
     buildConf("session.engine.launch.async")
       .doc("When opening kyuubi session, whether to launch backend engine asynchronously." +
@@ -671,7 +737,7 @@ object KyuubiConf {
         " as the backend engine will be created asynchronously.")
       .version("1.4.0")
       .booleanConf
-      .createWithDefault(false)
+      .createWithDefault(true)
 
   val SERVER_EXEC_POOL_SIZE: ConfigEntry[Int] =
     buildConf("backend.server.exec.pool.size")
@@ -792,7 +858,7 @@ object KyuubiConf {
 
   // [ZooKeeper Data Model]
   // (http://zookeeper.apache.org/doc/r3.7.0/zookeeperProgrammers.html#ch_zkDataModel)
-  private val validEngineSubdomain: Pattern = ("(?!^[\\u002e]{1,2}$)" +
+  private val validZookeeperSubPath: Pattern = ("(?!^[\\u002e]{1,2}$)" +
     "(^[\\u0020-\\u002e\\u0030-\\u007e\\u00a0-\\ud7ff\\uf900-\\uffef]{1,}$)").r.pattern
 
   @deprecated(s"using kyuubi.engine.share.level.subdomain instead", "1.4.0")
@@ -802,7 +868,7 @@ object KyuubiConf {
       .version("1.2.0")
       .stringConf
       .transform(_.toLowerCase(Locale.ROOT))
-      .checkValue(validEngineSubdomain.matcher(_).matches(), "must be valid zookeeper sub path.")
+      .checkValue(validZookeeperSubPath.matcher(_).matches(), "must be valid zookeeper sub path.")
       .createOptional
 
   val ENGINE_SHARE_LEVEL_SUBDOMAIN: ConfigEntry[Option[String]] =
@@ -815,13 +881,21 @@ object KyuubiConf {
       .version("1.4.0")
       .fallbackConf(ENGINE_SHARE_LEVEL_SUB_DOMAIN)
 
+  @deprecated(s"using ${FRONTEND_CONNECTION_URL_USE_HOSTNAME.key} instead, 1.5.0")
   val ENGINE_CONNECTION_URL_USE_HOSTNAME: ConfigEntry[Boolean] =
     buildConf("engine.connection.url.use.hostname")
-      .doc("When true, engine register with hostname to zookeeper. When spark run on k8s" +
+      .doc("(deprecated) " +
+        "When true, engine register with hostname to zookeeper. When spark run on k8s" +
         " with cluster mode, set to false to ensure that server can connect to engine")
       .version("1.3.0")
       .booleanConf
-      .createWithDefault(true)
+      .createWithDefault(false)
+
+  val FRONTEND_CONNECTION_URL_USE_HOSTNAME: ConfigEntry[Boolean] =
+    buildConf("frontend.connection.url.use.hostname")
+      .doc("When true, frontend services prefer hostname, otherwise, ip address")
+      .version("1.5.0")
+      .fallbackConf(ENGINE_CONNECTION_URL_USE_HOSTNAME)
 
   val ENGINE_SHARE_LEVEL: ConfigEntry[String] = buildConf("engine.share.level")
     .doc("Engines will be shared in different levels, available configs are: <ul>" +
@@ -853,6 +927,13 @@ object KyuubiConf {
     .checkValues(EngineType.values.map(_.toString))
     .createWithDefault(EngineType.SPARK_SQL.toString)
 
+  val ENGINE_POOL_NAME: ConfigEntry[String] = buildConf("engine.pool.name")
+    .doc("The name of engine pool.")
+    .version("1.5.0")
+    .stringConf
+    .checkValue(validZookeeperSubPath.matcher(_).matches(), "must be valid zookeeper sub path.")
+    .createWithDefault("engine-pool")
+
   val ENGINE_POOL_SIZE_THRESHOLD: ConfigEntry[Int] = buildConf("engine.pool.size.threshold")
     .doc("This parameter is introduced as a server-side parameter, " +
       "and controls the upper limit of the engine pool.")
@@ -982,10 +1063,10 @@ object KyuubiConf {
       .doc("A comma separated list of engine history loggers, where engine/session/operation etc" +
         " events go. We use spark logger by default.<ul>" +
         " <li>SPARK: the events will be written to the spark listener bus.</li>" +
-        s" <li>JSON: the events will be written to the location of" +
+        " <li>JSON: the events will be written to the location of" +
         s" ${ENGINE_EVENT_JSON_LOG_PATH.key}</li>" +
-        s" <li>JDBC: to be done</li>" +
-        s" <li>CUSTOM: to be done.</li></ul>")
+        " <li>JDBC: to be done</li>" +
+        " <li>CUSTOM: to be done.</li></ul>")
       .version("1.3.0")
       .stringConf
       .transform(_.toUpperCase(Locale.ROOT))
@@ -1025,6 +1106,59 @@ object KyuubiConf {
       .stringConf
       .createWithDefault("engine_operation_logs")
 
+  val ENGINE_SECURITY_ENABLED: ConfigEntry[Boolean] =
+    buildConf("engine.security.enabled")
+      .doc("Whether to enable the internal secure access between Kyuubi server and engine.")
+      .version("1.5.0")
+      .booleanConf
+      .createWithDefault(false)
+
+  val ENGINE_SECURITY_TOKEN_MAX_LIFETIME: ConfigEntry[Long] =
+    buildConf("engine.security.token.max.lifetime")
+      .doc("The max lifetime of the token used for secure access between Kyuubi server and engine.")
+      .version("1.5.0")
+      .timeConf
+      .createWithDefault(Duration.ofMinutes(10).toMillis)
+
+  val ENGINE_SECURITY_SECRET_PROVIDER: ConfigEntry[String] =
+    buildConf("engine.security.secret.provider")
+      .doc("The class used to manage the engine security secret. This class must be a " +
+        "subclass of EngineSecuritySecretProvider.")
+      .version("1.5.0")
+      .stringConf
+      .createWithDefault(
+        "org.apache.kyuubi.service.authentication.ZooKeeperEngineSecuritySecretProviderImpl")
+
+  val ENGINE_SECURITY_CRYPTO_KEY_LENGTH: ConfigEntry[Int] =
+    buildConf("engine.security.crypto.keyLength")
+      .doc("The length in bits of the encryption key to generate. " +
+        "Valid values are 128, 192 and 256")
+      .version("1.5.0")
+      .intConf
+      .checkValues(Set(128, 192, 256))
+      .createWithDefault(128)
+
+  val ENGINE_SECURITY_CRYPTO_IV_LENGTH: ConfigEntry[Int] =
+    buildConf("engine.security.crypto.ivLength")
+      .doc("Initial vector length, in bytes.")
+      .version("1.5.0")
+      .intConf
+      .createWithDefault(16)
+
+  val ENGINE_SECURITY_CRYPTO_KEY_ALGORITHM: ConfigEntry[String] =
+    buildConf("engine.security.crypto.keyAlgorithm")
+      .doc("The algorithm for generated secret keys.")
+      .version("1.5.0")
+      .stringConf
+      .createWithDefault("AES")
+
+  val ENGINE_SECURITY_CRYPTO_CIPHER_TRANSFORMATION: ConfigEntry[String] =
+    buildConf("engine.security.crypto.cipher")
+      .doc("The cipher transformation to use for encrypting engine access token.")
+      .version("1.5.0")
+      .stringConf
+      .createWithDefault("AES/CBC/PKCS5PADDING")
+
   val SESSION_NAME: OptionalConfigEntry[String] =
     buildConf("session.name")
       .doc("A human readable name of session and we use empty string by default. " +
@@ -1036,16 +1170,63 @@ object KyuubiConf {
 
   object OperationModes extends Enumeration {
     type OperationMode = Value
-    val PARSE, ANALYZE, OPTIMIZE, NONE = Value
+    val PARSE, ANALYZE, OPTIMIZE, PHYSICAL, EXECUTION, NONE = Value
   }
 
-  val OPERATION_PLAN_ONLY: ConfigEntry[String] =
+  val OPERATION_PLAN_ONLY_MODE: ConfigEntry[String] =
     buildConf("operation.plan.only.mode")
-      .doc("Whether to perform the statement in a PARSE, ANALYZE, OPTIMIZE only way without " +
-        "executing the query. When it is NONE, the statement will be fully executed")
+      .doc("Whether to perform the statement in a PARSE, ANALYZE, OPTIMIZE, PHYSICAL, EXECUTION " +
+        "only way without executing the query. When it is NONE, the statement will be fully " +
+        "executed")
       .version("1.4.0")
       .stringConf
       .transform(_.toUpperCase(Locale.ROOT))
       .checkValues(OperationModes.values.map(_.toString))
       .createWithDefault(OperationModes.NONE.toString)
+
+  val OPERATION_PLAN_ONLY_EXCLUDES: ConfigEntry[Seq[String]] =
+    buildConf("operation.plan.only.excludes")
+      .doc("Comma-separated list of query plan names, in the form of simple class names, i.e, " +
+        "for `set abc=xyz`, the value will be `SetCommand`. For those auxiliary plans, such as " +
+        "`switch databases`, `set properties`, or `create temporary view` e.t.c, " +
+        "which are used for setup evaluating environments for analyzing actual queries, " +
+        "we can use this config to exclude them and let them take effect. " +
+        s"See also ${OPERATION_PLAN_ONLY_MODE.key}.")
+      .version("1.5.0")
+      .stringConf
+      .toSequence()
+      .createWithDefault(Seq("ResetCommand", "SetCommand", "SetNamespaceCommand", "UseStatement"))
+
+  object OperationLanguages extends Enumeration {
+    type OperationLanguage = Value
+    val SQL, SCALA = Value
+  }
+
+  val OPERATION_LANGUAGE: ConfigEntry[String] =
+    buildConf("operation.language")
+      .doc("Choose a programing language for the following inputs" +
+        " <ul><li>SQL: (Default) Run all following statements as SQL queries.</li>" +
+        " <li>SCALA: Run all following input a scala codes</li></ul>")
+      .version("1.5.0")
+      .stringConf
+      .transform(_.toUpperCase(Locale.ROOT))
+      .checkValues(OperationLanguages.values.map(_.toString))
+      .createWithDefault(OperationLanguages.SQL.toString)
+
+  val SESSION_CONF_ADVISOR: OptionalConfigEntry[String] =
+    buildConf("session.conf.advisor")
+      .doc("A config advisor plugin for Kyuubi Server. This plugin can provide some custom " +
+        "configs for different user or session configs and overwrite the session configs before " +
+        "open a new session. This config value should be a class which is a child of " +
+        "'org.apache.kyuubi.plugin.SessionConfAdvisor' which has zero-arg constructor.")
+      .version("1.5.0")
+      .stringConf
+      .createOptional
+
+  val SERVER_NAME: OptionalConfigEntry[String] =
+    buildConf("server.name")
+      .doc("The name of Kyuubi Server.")
+      .version("1.5.0")
+      .stringConf
+      .createOptional
 }
