@@ -19,6 +19,7 @@ package org.apache.kyuubi.operation
 
 import java.util.concurrent.Future
 
+import org.apache.commons.lang3.StringUtils
 import org.apache.hive.service.rpc.thrift.{TProtocolVersion, TRowSet, TTableSchema}
 
 import org.apache.kyuubi.{KyuubiSQLException, Logging}
@@ -38,7 +39,7 @@ abstract class AbstractOperation(opType: OperationType, session: Session)
     session.sessionManager.getConf.get(OPERATION_IDLE_TIMEOUT)
   }
 
-  final protected val statementId = handle.identifier.toString
+  final private[kyuubi] val statementId = handle.identifier.toString
 
   override def getOperationLog: Option[OperationLog] = None
 
@@ -58,7 +59,9 @@ abstract class AbstractOperation(opType: OperationType, session: Session)
 
   def getBackgroundHandle: Future[_] = _backgroundHandle
 
-  protected def statement: String = opType.toString
+  def statement: String = opType.toString
+
+  def redactedStatement: String = statement
 
   protected def setHasResultSet(hasResultSet: Boolean): Unit = {
     this.hasResultSet = hasResultSet
@@ -80,7 +83,7 @@ abstract class AbstractOperation(opType: OperationType, session: Session)
       case _ =>
     }
     info(s"Processing ${session.user}'s query[$statementId]: ${state.name} -> ${newState.name}," +
-      s" statement: $statement$timeCost")
+      s" statement: $redactedStatement$timeCost")
     state = newState
     lastAccessTime = System.currentTimeMillis()
   }
@@ -143,6 +146,29 @@ abstract class AbstractOperation(opType: OperationType, session: Session)
 
   override def getNextRowSet(order: FetchOrientation, rowSetSize: Int): TRowSet
 
+  /**
+   * convert SQL 'like' pattern to a Java regular expression.
+   *
+   * Underscores (_) are converted to '.' and percent signs (%) are converted to '.*'.
+   *
+   * (referred to Spark's implementation: convertPattern function in file MetadataOperation.java)
+   *
+   * @param input the SQL pattern to convert
+   * @return the equivalent Java regular expression of the pattern
+   */
+  protected def toJavaRegex(input: String): String = {
+    val res =
+      if (StringUtils.isEmpty(input) || input == "*") {
+        "%"
+      } else {
+        input
+      }
+    val wStr = ".*"
+    res
+      .replaceAll("([^\\\\])%", "$1" + wStr).replaceAll("\\\\%", "%").replaceAll("^%", wStr)
+      .replaceAll("([^\\\\])_", "$1.").replaceAll("\\\\_", "_").replaceAll("^_", ".")
+  }
+
   override def getSession: Session = session
 
   override def getHandle: OperationHandle = handle
