@@ -277,7 +277,8 @@ class ServiceControlCliSuite extends KyuubiFunSuite with TestPrematureExit {
       namespace,
       "--user",
       user)
-    assert(getZkNamespace(new ServiceControlCliArguments(arg2)) == s"/${namespace}_USER/$user")
+    assert(getZkNamespace(new ServiceControlCliArguments(arg2)) ==
+      s"/${namespace}_USER_SPARK_SQL/$user/default")
   }
 
   test("test list zk service nodes info") {
@@ -404,4 +405,62 @@ class ServiceControlCliSuite extends KyuubiFunSuite with TestPrematureExit {
       testPrematureExit(args, getRenderedNodesInfoWithoutTitle(expectedNodes, true))
     }
   }
+
+  test("test get zk namespace for different engine type and subdomain") {
+    val arg1 = Array(
+      "list",
+      "engine",
+      "--zk-quorum",
+      zkServer.getConnectString,
+      "--namespace",
+      namespace,
+      "--user",
+      user)
+    assert(getZkNamespace(new ServiceControlCliArguments(arg1)) ==
+      s"/${namespace}_USER_SPARK_SQL/$user/default")
+
+    val arg2 = Array(
+      "list",
+      "engine",
+      "--zk-quorum",
+      zkServer.getConnectString,
+      "--namespace",
+      namespace,
+      "--user",
+      user,
+      "--engine-type",
+      "FLINK_SQL")
+    assert(getZkNamespace(new ServiceControlCliArguments(arg2)) ==
+      s"/${namespace}_USER_FLINK_SQL/$user/default")
+
+    val arg3 = Array(
+      "list",
+      "engine",
+      "--zk-quorum",
+      zkServer.getConnectString,
+      "--namespace",
+      namespace,
+      "--user",
+      user,
+      "--engine-type",
+      "TRINO")
+    assert(getZkNamespace(new ServiceControlCliArguments(arg3)) ==
+      s"/${namespace}_USER_TRINO/$user/default")
+
+    val arg4 = Array(
+      "list",
+      "engine",
+      "--zk-quorum",
+      zkServer.getConnectString,
+      "--namespace",
+      namespace,
+      "--user",
+      user,
+      "--engine-type",
+      "SPARK_SQL",
+      "--engine-subdomain",
+      "sub_1")
+    assert(getZkNamespace(new ServiceControlCliArguments(arg4)) ==
+      s"/${namespace}_USER_SPARK_SQL/$user/sub_1")
+  }
 }
