@@ -29,12 +29,15 @@ import org.apache.curator.framework.recipes.locks.InterProcessSemaphoreMutex
 import org.apache.curator.utils.ZKPaths
 import org.apache.hadoop.security.UserGroupInformation
 
-import org.apache.kyuubi.{KyuubiSQLException, Logging, Utils}
+import org.apache.kyuubi.{KYUUBI_VERSION, KyuubiSQLException, Logging, Utils}
 import org.apache.kyuubi.config.KyuubiConf
 import org.apache.kyuubi.config.KyuubiConf._
-import org.apache.kyuubi.engine.EngineType.{EngineType, SPARK_SQL}
+import org.apache.kyuubi.config.KyuubiReservedKeys.KYUUBI_ENGINE_SUBMIT_TIME_KEY
+import org.apache.kyuubi.engine.EngineType.{EngineType, FLINK_SQL, SPARK_SQL, TRINO}
 import org.apache.kyuubi.engine.ShareLevel.{CONNECTION, GROUP, SERVER, ShareLevel}
+import org.apache.kyuubi.engine.flink.FlinkProcessBuilder
 import org.apache.kyuubi.engine.spark.SparkProcessBuilder
+import org.apache.kyuubi.engine.trino.TrinoProcessBuilder
 import org.apache.kyuubi.ha.HighAvailabilityConf.HA_ZK_ENGINE_REF_ID
 import org.apache.kyuubi.ha.HighAvailabilityConf.HA_ZK_NAMESPACE
 import org.apache.kyuubi.ha.client.ServiceDiscovery.getEngineByRefId
@@ -70,6 +73,8 @@ private[kyuubi] class EngineRef(
 
   private val clientPoolSize: Int = conf.get(ENGINE_POOL_SIZE)
 
+  private val clientPoolName: String = conf.get(ENGINE_POOL_NAME)
+
   @VisibleForTesting
   private[kyuubi] val subdomain: String = conf.get(ENGINE_SHARE_LEVEL_SUBDOMAIN) match {
     case Some(_subdomain) => _subdomain
@@ -81,7 +86,7 @@ private[kyuubi] class EngineRef(
       }
       // TODO: Currently, we use random policy, and later we can add a sequential policy,
       //  such as AtomicInteger % poolSize.
-      "engine-pool-" + Random.nextInt(poolSize)
+      s"$clientPoolName-${Random.nextInt(poolSize)}"
     case _ => "default" // [KYUUBI #1293]
   }
 
@@ -127,7 +132,7 @@ private[kyuubi] class EngineRef(
    */
   @VisibleForTesting
   private[kyuubi] lazy val engineSpace: String = {
-    val commonParent = s"${serverSpace}_${shareLevel}_$engineType"
+    val commonParent = s"${serverSpace}_${KYUUBI_VERSION}_${shareLevel}_$engineType"
     shareLevel match {
       case CONNECTION => ZKPaths.makePath(commonParent, appUser, engineRefId)
       case _ => ZKPaths.makePath(commonParent, appUser, subdomain)
@@ -174,6 +179,8 @@ private[kyuubi] class EngineRef(
 
     conf.set(HA_ZK_NAMESPACE, engineSpace)
     conf.set(HA_ZK_ENGINE_REF_ID, engineRefId)
+    val started = System.currentTimeMillis()
+    conf.set(KYUUBI_ENGINE_SUBMIT_TIME_KEY, String.valueOf(started))
     val builder = engineType match {
       case SPARK_SQL =>
         conf.setIfMissing(SparkProcessBuilder.APP_KEY, defaultEngineName)
@@ -182,13 +189,23 @@ private[kyuubi] class EngineRef(
           SparkProcessBuilder.TAG_KEY,
           conf.getOption(SparkProcessBuilder.TAG_KEY).map(_ + ",").getOrElse("") + "KYUUBI")
         new SparkProcessBuilder(appUser, conf, extraEngineLog)
-      case _ => throw new UnsupportedOperationException(s"Unsupported engine type: ${engineType}")
+      case FLINK_SQL =>
+        conf.setIfMissing(FlinkProcessBuilder.APP_KEY, defaultEngineName)
+        // tag is a seq type with comma-separated
+        conf.set(
+          FlinkProcessBuilder.TAG_KEY,
+          conf.getOption(FlinkProcessBuilder.TAG_KEY).map(_ + ",").getOrElse("") + "KYUUBI")
+        conf.set(HA_ZK_NAMESPACE, engineSpace)
+        conf.set(HA_ZK_ENGINE_REF_ID, engineRefId)
+        new FlinkProcessBuilder(appUser, conf, extraEngineLog)
+      case TRINO =>
+        new TrinoProcessBuilder(appUser, conf, extraEngineLog)
     }
+
     MetricsSystem.tracing(_.incCount(ENGINE_TOTAL))
     try {
       info(s"Launching engine:\n$builder")
       val process = builder.start
-      val started = System.currentTimeMillis()
       var exitValue: Option[Int] = None
       while (engineRef.isEmpty) {
         if (exitValue.isEmpty && process.waitFor(1, TimeUnit.SECONDS)) {
