@@ -25,22 +25,20 @@ import org.apache.thrift.TException
 
 import org.apache.kyuubi.KyuubiSQLException
 import org.apache.kyuubi.config.KyuubiConf
-import org.apache.kyuubi.events.KyuubiStatementEvent
-import org.apache.kyuubi.metrics.MetricsConstants._
-import org.apache.kyuubi.metrics.MetricsSystem
+import org.apache.kyuubi.events.{EventLogging, KyuubiOperationEvent}
 import org.apache.kyuubi.operation.FetchOrientation.FETCH_NEXT
 import org.apache.kyuubi.operation.OperationState.OperationState
 import org.apache.kyuubi.operation.log.OperationLog
-import org.apache.kyuubi.server.EventLoggingService
 import org.apache.kyuubi.session.{KyuubiSessionImpl, KyuubiSessionManager, Session}
 
 class ExecuteStatement(
     session: Session,
     override val statement: String,
+    confOverlay: Map[String, String],
     override val shouldRunAsync: Boolean,
     queryTimeout: Long)
   extends KyuubiOperation(OperationType.EXECUTE_STATEMENT, session) {
-  EventLoggingService.onEvent(KyuubiStatementEvent(this))
+  EventLogging.onEvent(KyuubiOperationEvent(this))
 
   final private val _operationLog: OperationLog =
     if (shouldRunAsync) {
@@ -67,14 +65,11 @@ class ExecuteStatement(
 
   private def executeStatement(): Unit = {
     try {
-      MetricsSystem.tracing { ms =>
-        ms.incCount(STATEMENT_OPEN)
-        ms.incCount(STATEMENT_TOTAL)
-      }
       // We need to avoid executing query in sync mode, because there is no heartbeat mechanism
       // in thrift protocol, in sync mode, we cannot distinguish between long-run query and
       // engine crash without response before socket read timeout.
-      _remoteOpHandle = client.executeStatement(statement, true, queryTimeout)
+      _remoteOpHandle = client.executeStatement(statement, confOverlay, true, queryTimeout)
+      setHasResultSet(_remoteOpHandle.isHasResultSet)
     } catch onError()
   }
 
@@ -179,11 +174,10 @@ class ExecuteStatement(
 
   override def setState(newState: OperationState): Unit = {
     super.setState(newState)
-    EventLoggingService.onEvent(KyuubiStatementEvent(this))
+    EventLogging.onEvent(KyuubiOperationEvent(this))
   }
 
   override def close(): Unit = {
-    MetricsSystem.tracing(_.decCount(STATEMENT_OPEN))
     super.close()
   }
 }
