@@ -26,9 +26,10 @@ import org.apache.thrift.TException
 import org.apache.thrift.transport.TTransportException
 
 import org.apache.kyuubi.{KyuubiSQLException, Utils}
-import org.apache.kyuubi.metrics.MetricsConstants.STATEMENT_FAIL
+import org.apache.kyuubi.metrics.MetricsConstants.{OPERATION_FAIL, OPERATION_OPEN, OPERATION_STATE, OPERATION_TOTAL}
 import org.apache.kyuubi.metrics.MetricsSystem
 import org.apache.kyuubi.operation.FetchOrientation.FetchOrientation
+import org.apache.kyuubi.operation.OperationState.OperationState
 import org.apache.kyuubi.operation.OperationType.OperationType
 import org.apache.kyuubi.session.{KyuubiSessionImpl, Session}
 import org.apache.kyuubi.util.ThriftUtils
@@ -36,6 +37,18 @@ import org.apache.kyuubi.util.ThriftUtils
 abstract class KyuubiOperation(opType: OperationType, session: Session)
   extends AbstractOperation(opType, session) {
 
+  private val opTypeName = (opType match {
+    case OperationType.UNKNOWN_OPERATION => statement
+    case _ => opType.toString
+  }).toLowerCase
+
+  MetricsSystem.tracing { ms =>
+    ms.incCount(MetricRegistry.name(OPERATION_OPEN, opTypeName))
+    ms.incCount(MetricRegistry.name(OPERATION_TOTAL, opTypeName))
+    ms.incCount(MetricRegistry.name(OPERATION_TOTAL))
+    ms.markMeter(MetricRegistry.name(OPERATION_STATE, state.toString.toLowerCase))
+  }
+
   protected[operation] lazy val client = session.asInstanceOf[KyuubiSessionImpl].client
 
   @volatile protected var _remoteOpHandle: TOperationHandle = _
@@ -53,9 +66,8 @@ abstract class KyuubiOperation(opType: OperationType, session: Session)
           warn(s"Ignore exception in terminal state with $statementId: $e")
         } else {
           val errorType = e.getClass.getSimpleName
-          MetricsSystem.tracing {
-            _.incCount(MetricRegistry.name(STATEMENT_FAIL, errorType))
-          }
+          MetricsSystem.tracing(_.incCount(
+            MetricRegistry.name(OPERATION_FAIL, opTypeName, errorType)))
           val ke = e match {
             case kse: KyuubiSQLException => kse
             case te: TTransportException
@@ -91,6 +103,7 @@ abstract class KyuubiOperation(opType: OperationType, session: Session)
   override def cancel(): Unit = state.synchronized {
     if (!isClosedOrCanceled) {
       setState(OperationState.CANCELED)
+      MetricsSystem.tracing(_.decCount(MetricRegistry.name(OPERATION_OPEN, opTypeName)))
       if (_remoteOpHandle != null) {
         try {
           client.cancelOperation(_remoteOpHandle)
@@ -105,6 +118,7 @@ abstract class KyuubiOperation(opType: OperationType, session: Session)
   override def close(): Unit = state.synchronized {
     if (!isClosedOrCanceled) {
       setState(OperationState.CLOSED)
+      MetricsSystem.tracing(_.decCount(MetricRegistry.name(OPERATION_OPEN, opTypeName)))
       if (_remoteOpHandle != null) {
         try {
           getOperationLog.foreach(_.close())
@@ -146,4 +160,10 @@ abstract class KyuubiOperation(opType: OperationType, session: Session)
   }
 
   override def shouldRunAsync: Boolean = false
+
+  override def setState(newState: OperationState): Unit = {
+    super.setState(newState)
+    MetricsSystem.tracing(
+      _.markMeter(MetricRegistry.name(OPERATION_STATE, newState.toString.toLowerCase)))
+  }
 }
